package cn.fw.hermes.controller.api.service;

import cn.fw.data.base.domain.common.Message;
import cn.fw.hermes.sdk.api.ImSendMessage;
import cn.fw.hermes.sdk.api.para.MsgPara;
import cn.fw.hermes.sdk.api.para.MsgParamCondition;
import cn.fw.hermes.service.biz.MessageBizService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.hermes.common.constant.MessageStr.SAVE_FAILURE;

/**
 * @Author: Chenery
 * @Date: 2021/2/2 15:59
 */
@RestController
@Slf4j
@RequiredArgsConstructor
public class ImSendMsgApiServiceImpl implements ImSendMessage {

    private final MessageBizService messageBizService;


    @Override
    @PostMapping("/api/msg/asynSendMsg")
    public Message<String> asynSendMsg(@RequestBody MsgParamCondition condition) {
        final String msg = "发送异步消息[/api/msg/asynSendMsg]";
        try {
            log.info("{} params[{}]", msg, condition);
            String messageId = messageBizService.asynSendMsg(condition);
            log.info("{} 成功 ：消息ID [{}]", msg, messageId);
            return success(messageId);
        } catch (Exception ex) {
            handleException(ex, e -> log.info("{} 失败：result: [{}]", msg, e));
            return failureWithMessage(SAVE_FAILURE);
        }
    }

    @Override
    @PostMapping("/api/msg/synSendMsg")
    public Message<Boolean> synSendMsg(@RequestBody MsgParamCondition condition) {
        final String msg = "发送同步消息[/api/msg/synSendMsg]";
        try {
            log.info("{} params[{}]", msg, condition);
            Boolean result = messageBizService.synSendMsg(condition);
            return success(result, data -> log.info("{} 成功 ：消息发送结果 [{}]", msg, result));
        } catch (Exception ex) {
            handleException(ex, e -> log.info("{} 失败：result: [{}]", msg, e));
            return failureWithMessage(SAVE_FAILURE);
        }
    }

    @Override
    @PostMapping("/api/im/sendMsg")
    public Message<Integer> sendMsg(@RequestBody MsgPara msgPara) {
        final String msg = "发送消息[/app/msg/sendMsg]";
        try {
            log.info("{} : params[{}]", msg, msgPara);
            Boolean result = messageBizService.synSendMsg(msgPara);
            return success(result ? 1 : 0, data -> log.info("{} 成功 ：result[{}]", msg, result));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{} 失败: result:[{}]", msg, e));
            return failureWithMessage(SAVE_FAILURE);
        }

    }


}
