package cn.fw.hermes.task;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.hermes.common.constant.RedisKey;
import cn.fw.hermes.common.utils.DateUtils;
import cn.fw.hermes.service.biz.ChatRecordBizService;
import java.util.Date;
import java.util.concurrent.locks.Lock;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * 定时拉取IM聊天记录 默认每晚凌晨执行
 *
 * @Author: Chenery
 * @Date: 2021/2/21 17:32
 */
@Component
@RequiredArgsConstructor
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class PullChatRecordTask {

    private final ChatRecordBizService chatRecordBizService;
    private final DistributedLocker distributedLocker;
    private final static String[] hurs = new String[]{"00", "01", "02", "03", "04", "05", "06",
            "07", "08", "09", "10",
            "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23"};

    /**
     * 每天凌晨1点执行一次  todo  请勿修改定时时间
     */
    // @Scheduled(cron = "0 */1 * * * ?")
    @Scheduled(cron = "0 0 1 * * ?")
    public void pullChatRecord() {
        final String msg = "同步IM聊天记录任务";
        log.info("{} 开始，同步时间：{}", msg,DateUtils.getNow());
        Lock lock = distributedLocker.lock(RedisKey.PULL_CHAT_RECORD.getCode(), 1000 * 60);
        try {
            Date date = DateUtils.addSomeDay(new Date(), -1);
            String dateStr = DateUtils.format(date, "yyyyMMdd");
            for (String hour : hurs) {
                String timePoint = dateStr + hour;
                chatRecordBizService.pullChatRecord(timePoint);
            }

        } catch (Exception e) {
            log.error("{} 执行失败 错误信息：[{}]", msg, e);
        } finally {
            distributedLocker.unlock(lock);
        }


    }

}
